#version 410 compatibility

uniform int tess_level;

in vec3 normal;
in vec3 vpos;
in vec2 uv;


void main()
{
	vec3 ldir = gl_LightSource[0].position.xyz - vpos;

	vec3 n = normalize(normal);
	vec3 v = -normalize(vpos);
	vec3 l = normalize(ldir);
	vec3 h = normalize(l + v);

	float ndotl = max(dot(n, l), 0.0);
	float ndoth = max(dot(n, h), 0.0);

	vec3 ka = gl_FrontMaterial.ambient.xyz;
	vec3 kd = gl_FrontMaterial.diffuse.xyz;
	vec3 ks = gl_FrontMaterial.specular.xyz;
	float shin = gl_FrontMaterial.shininess;

	vec3 diffuse = kd * ndotl * gl_LightSource[0].diffuse.xyz;
	vec3 specular = ks * pow(ndoth, shin) * gl_LightSource[0].specular.xyz;
	vec3 ambient = ka * gl_LightModel.ambient.xyz;
	vec3 color = ambient + diffuse + specular;

	vec2 tess_uv = mod(uv * float(tess_level), 1.0);

#define STEPSZ	0.1
	float wire = smoothstep(0.0, STEPSZ, tess_uv.x) * smoothstep(1.0, 1.0 - STEPSZ, tess_uv.x);
	wire *= smoothstep(0.0, STEPSZ, tess_uv.y) * smoothstep(1.0, 1.0 - STEPSZ, tess_uv.y);

	gl_FragColor = vec4(mix(vec3(0.0, 0.0, 0.0), color, wire), gl_FrontMaterial.diffuse.w);
}
