#version 410 compatibility

layout(quads, ccw) in;

out vec3 normal;
out vec3 vpos;
out vec2 uv;

uniform vec3 norm_scale;

vec3 bezier_patch(float u, float v);
vec3 bezier_patch_norm(float u, float v);
float bernstein(int i, float x);

void main()
{
	vec3 pos = bezier_patch(gl_TessCoord.x, gl_TessCoord.y);
	normal = gl_NormalMatrix * bezier_patch_norm(gl_TessCoord.x, gl_TessCoord.y);

	uv = gl_TessCoord.xy;

	gl_Position = gl_ModelViewProjectionMatrix * vec4(pos, 1.0);
	vpos = (gl_ModelViewMatrix * vec4(pos, 1.0)).xyz;
}

vec3 bezier_patch(float u, float v)
{
	int i, j;
	vec3 res = vec3(0.0, 0.0, 0.0);

	for(j=0; j<4; j++) {
		for(i=0; i<4; i++) {
			float bu = bernstein(i, u);
			float bv = bernstein(j, v);

			res += gl_in[j * 4 + i].gl_Position.xyz * bu * bv;
		}
	}
	return res;
}

#define DT 0.0001
vec3 bezier_patch_norm(float u, float v)
{
	vec3 tang = bezier_patch(u + DT, v) - bezier_patch(u - DT, v);
	vec3 bitan = bezier_patch(u, v + DT) - bezier_patch(u, v - DT);
	return cross(tang, bitan) * norm_scale;
}

float bernstein(int i, float x)
{
	float invx = 1.0 - x;

	if(i == 0) {
		return invx * invx * invx;
	}
	if(i == 1) {
		return 3 * x * invx * invx;
	}
	if(i == 2) {
		return 3 * x * x * invx;
	}
	return x * x * x;
}
