#include <math.h>
#include "vmath.h"

static float bernstein(int i, float x);


struct vec3 v3_add(struct vec3 a, struct vec3 b)
{
	a.x += b.x;
	a.y += b.y;
	a.z += b.z;
	return a;
}

struct vec3 v3_sub(struct vec3 a, struct vec3 b)
{
	a.x -= b.x;
	a.y -= b.y;
	a.z -= b.z;
	return a;
}

struct vec3 v3_cross(struct vec3 a, struct vec3 b)
{
	struct vec3 res;
	res.x = a.y * b.z - a.z * b.y;
	res.y = a.z * b.x - a.x * b.z;
	res.z = a.x * b.y - a.y * b.x;
	return res;
}

struct vec3 v3_normalize(struct vec3 v)
{
	float len = sqrt(v.x * v.x + v.y * v.y + v.z * v.z);
	v.x /= len;
	v.y /= len;
	v.z /= len;
	return v;
}

struct vec3 bezier_patch(struct vec3 *cp, float u, float v)
{
	int i, j;
	struct vec3 res = {0, 0, 0};

	for(j=0; j<4; j++) {
		for(i=0; i<4; i++) {
			float bu = bernstein(i, u);
			float bv = bernstein(j, v);

			res.x += cp->x * bu * bv;
			res.y += cp->y * bu * bv;
			res.z += cp->z * bu * bv;

			cp++;
		}
	}
	return res;
}

#define DT	0.001

struct vec3 bezier_patch_norm(struct vec3 *cp, float u, float v)
{
	struct vec3 tang, bitan, norm;

	tang = v3_sub(bezier_patch(cp, u + DT, v), bezier_patch(cp, u - DT, v));
	bitan = v3_sub(bezier_patch(cp, u, v + DT), bezier_patch(cp, u, v - DT));
	norm = v3_cross(tang, bitan);

	return v3_normalize(norm);
}



static float bernstein(int i, float x)
{
	float invx = 1.0 - x;

	switch(i) {
	case 0:
		return invx * invx * invx;
	case 1:
		return 3 * x * invx * invx;
	case 2:
		return 3 * x * x * invx;
	case 3:
		return x * x * x;
	default:
		break;
	}
	return 0;
}
